import { safeLoadBtModule } from "..";
import { BtModule, btModulesLoadConfig } from "../types";
import assetLoader from "@braintree/asset-loader";

jest.mock("@braintree/asset-loader", () => {
  return {
    loadScript: jest.fn(),
    loadStylesheet: jest.fn(),
  };
});

describe("safeLoadBtModule", () => {
  it("should attempt to load BT module", async () => {
    const { id, script } = btModulesLoadConfig.client;
    const version = "3.97.2";

    await safeLoadBtModule(btModulesLoadConfig.client, version);

    const scriptSrc = `https://js.braintreegateway.com/web/${version}/js/${script}`;
    const loadScriptOptions = {
      id: `${id}-${version}`,
      src: scriptSrc,
    };

    expect(assetLoader.loadScript).toHaveBeenCalledWith(loadScriptOptions);
  });

  it("should not load BT module if same version already exists", async () => {
    const version = "3.97.2";

    window.braintree = {
      [BtModule.Client]: {
        create: jest.fn(),
        VERSION: version,
      },
    };

    const loadResp = await safeLoadBtModule(
      btModulesLoadConfig.client,
      version
    );

    expect(assetLoader.loadScript).not.toHaveBeenCalled();
    expect(loadResp).toBe(true);
  });

  it("should not load BT module and return error if module on different version already exists", async () => {
    const version = "3.97.2";

    window.braintree = {
      [BtModule.Client]: {
        create: jest.fn(),
        VERSION: "3.96.0",
      },
    };

    try {
      await safeLoadBtModule(btModulesLoadConfig.client, version);
    } catch (error) {
      expect(error).toStrictEqual(
        new Error(
          "client already loaded with version 3.96.0 cannot load version 3.97.2"
        )
      );
    }

    expect(assetLoader.loadScript).not.toHaveBeenCalled();
  });

  it("should not load BT module and return error if no version specified", async () => {
    const version = "";

    window.braintree = {
      [BtModule.Client]: {
        create: jest.fn(),
        VERSION: "3.96.0",
      },
    };

    try {
      await safeLoadBtModule(btModulesLoadConfig.client, version);
    } catch (error) {
      expect(error).toStrictEqual(
        new Error("Attempted to load client without specifying version")
      );
    }

    expect(assetLoader.loadScript).not.toHaveBeenCalled();
  });
});
